#include "DisplayClusterInputDevice.h"

#include "CoreMinimal.h"
#include "Features/IModularFeatures.h"
#include "Input/IDisplayClusterInputManager.h"
#include "IDisplayCluster.h"

#include "DisplayClusterKeys.h"

#define LOCTEXT_NAMESPACE "DisplayClusterInput"

FDisplayClusterInputDevice::FDisplayClusterInputDevice(const TSharedRef<FGenericApplicationMessageHandler>& InMessageHandler) : MessageHandler(InMessageHandler)
{
  AxisIndices   = TArray<const FKey*>
  {
    &FDisplayClusterKeys::AxisX,
    &FDisplayClusterKeys::AxisY
  };
  ButtonIndices = TArray<const FKey*> 
  { 
    &FDisplayClusterKeys::Trigger, 
    &FDisplayClusterKeys::Action1,
    &FDisplayClusterKeys::Action2,
    &FDisplayClusterKeys::Action3,
    &FDisplayClusterKeys::Action4,
    &FDisplayClusterKeys::Action5,
    &FDisplayClusterKeys::Action6,
    &FDisplayClusterKeys::Action7,
    &FDisplayClusterKeys::Action8,
    &FDisplayClusterKeys::Action9
  };
}
FDisplayClusterInputDevice::~FDisplayClusterInputDevice()
{

}

void FDisplayClusterInputDevice::PreInit             ()
{
  EKeys::AddMenuCategoryDisplayInfo("DisplayCluster", LOCTEXT("DisplayClusterSubCategory", "Display Cluster"), TEXT("GraphEditor.PadEvent_16x"));

  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Trigger, LOCTEXT("DisplayClusterTrigger", "Display Cluster Trigger" ), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action1, LOCTEXT("DisplayClusterAction1", "Display Cluster Action 1"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action2, LOCTEXT("DisplayClusterAction2", "Display Cluster Action 2"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action3, LOCTEXT("DisplayClusterAction3", "Display Cluster Action 3"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action4, LOCTEXT("DisplayClusterAction4", "Display Cluster Action 4"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action5, LOCTEXT("DisplayClusterAction5", "Display Cluster Action 5"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action6, LOCTEXT("DisplayClusterAction6", "Display Cluster Action 6"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action7, LOCTEXT("DisplayClusterAction7", "Display Cluster Action 7"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action8, LOCTEXT("DisplayClusterAction8", "Display Cluster Action 8"), FKeyDetails::GamepadKey, "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::Action9, LOCTEXT("DisplayClusterAction9", "Display Cluster Action 9"), FKeyDetails::GamepadKey, "DisplayCluster"));

  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::AxisX  , LOCTEXT("DisplayClusterAxisX"  , "Display Cluster X Axis"  ), FKeyDetails::FloatAxis , "DisplayCluster"));
  EKeys::AddKey(FKeyDetails(FDisplayClusterKeys::AxisY  , LOCTEXT("DisplayClusterAxisY"  , "Display Cluster Y Axis"  ), FKeyDetails::FloatAxis , "DisplayCluster"));
}

bool FDisplayClusterInputDevice::Exec                (UWorld* InWorld, const TCHAR* Cmd, FOutputDevice& Ar)
{
  return false;
}                 
void FDisplayClusterInputDevice::SendControllerEvents()
{
  if (!IDisplayCluster::IsAvailable()) return;

  auto InputManager = IDisplayCluster::Get().GetInputMgr();
  
  TArray<FString> AxisDeviceIds;
  InputManager->GetAxisDeviceIds(AxisDeviceIds);
  for (auto i = 0; i < AxisDeviceIds.Num(); ++i)
  {
    for (auto j = 0; j < AxisIndices.Num(); ++j)
    {
      float Value;
      InputManager->GetAxis(AxisDeviceIds[i], j, Value);
      if (Value != 0.0f) MessageHandler->OnControllerAnalog(AxisIndices[j]->GetFName(), i, Value);
    }
  }

  TArray<FString> ButtonDeviceIds;
  InputManager->GetButtonDeviceIds(ButtonDeviceIds);
  for (auto i = 0; i < ButtonDeviceIds.Num(); ++i)
  {
    for (auto j = 0; j < ButtonIndices.Num(); ++j)
    {
      bool Pressed, Released;
      InputManager->WasButtonPressed (ButtonDeviceIds[i], j, Pressed );
      InputManager->WasButtonReleased(ButtonDeviceIds[i], j, Released);
      if (Pressed ) MessageHandler->OnControllerButtonPressed (ButtonIndices[j]->GetFName(), i, false);
      if (Released) MessageHandler->OnControllerButtonReleased(ButtonIndices[j]->GetFName(), i, false);
    }
  }
}                 
void FDisplayClusterInputDevice::SetChannelValue     (int32 ControllerId, FForceFeedbackChannelType ChannelType, float Value)
{
  // Intentionally empty (no haptic support).
} 
void FDisplayClusterInputDevice::SetChannelValues    (int32 ControllerId, const FForceFeedbackValues& Values)
{
  // Intentionally empty (no haptic support).
}
void FDisplayClusterInputDevice::SetMessageHandler   (const TSharedRef<FGenericApplicationMessageHandler>& InMessageHandler)
{
  MessageHandler = InMessageHandler;
}
void FDisplayClusterInputDevice::Tick                (float DeltaTime)
{
  // Intentionally empty (everything is handled in SendControllerEvents above).
}

#undef LOCTEXT_NAMESPACE